@@ -47,6 +47,7 @@ typedef struct {
     IActiveScriptProperty        IActiveScriptProperty_iface;
     IObjectSafety                IObjectSafety_iface;
     IVariantChangeType           IVariantChangeType_iface;
+    IWineJScript                 IWineJScript_iface;
 
     LONG ref;
 
@@ -669,6 +670,9 @@ static HRESULT WINAPI JScript_QueryInterface(IActiveScript *iface, REFIID riid,
     }else if(IsEqualGUID(riid, &IID_IVariantChangeType)) {
         TRACE("(%p)->(IID_IVariantChangeType %p)\n", This, ppv);
         *ppv = &This->IVariantChangeType_iface;
+    }else if(IsEqualGUID(riid, &IID_IWineJScript)) {
+        TRACE("(%p)->(IID_IWineJScript %p)\n", This, ppv);
+        *ppv = &This->IWineJScript_iface;
     }
 
     if(*ppv) {
@@ -1424,6 +1428,43 @@ static const IVariantChangeTypeVtbl VariantChangeTypeVtbl = {
     VariantChangeType_ChangeType
 };
 
+static inline JScript *impl_from_IWineJScript(IWineJScript *iface)
+{
+    return CONTAINING_RECORD(iface, JScript, IWineJScript_iface);
+}
+
+static HRESULT WINAPI WineJScript_QueryInterface(IWineJScript *iface, REFIID riid, void **ppv)
+{
+    JScript *This = impl_from_IWineJScript(iface);
+    return IActiveScript_QueryInterface(&This->IActiveScript_iface, riid, ppv);
+}
+
+static ULONG WINAPI WineJScript_AddRef(IWineJScript *iface)
+{
+    JScript *This = impl_from_IWineJScript(iface);
+    return IActiveScript_AddRef(&This->IActiveScript_iface);
+}
+
+static ULONG WINAPI WineJScript_Release(IWineJScript *iface)
+{
+    JScript *This = impl_from_IWineJScript(iface);
+    return IActiveScript_Release(&This->IActiveScript_iface);
+}
+
+static HRESULT WINAPI WineJScript_InitHostObject(IWineJScript *iface, IWineJSDispatchHost *host_obj,
+                                                 IWineJSDispatch **ret)
+{
+    JScript *This = impl_from_IWineJScript(iface);
+    return init_host_object(This->ctx, host_obj, ret);
+}
+
+static const IWineJScriptVtbl WineJScriptVtbl = {
+    WineJScript_QueryInterface,
+    WineJScript_AddRef,
+    WineJScript_Release,
+    WineJScript_InitHostObject,
+};
+
 HRESULT create_jscript_object(BOOL is_encode, REFIID riid, void **ppv)
 {
     JScript *ret;
@@ -1441,6 +1482,7 @@ HRESULT create_jscript_object(BOOL is_encode, REFIID riid, void **ppv)
     ret->IActiveScriptProperty_iface.lpVtbl = &JScriptPropertyVtbl;
     ret->IObjectSafety_iface.lpVtbl = &JScriptSafetyVtbl;
     ret->IVariantChangeType_iface.lpVtbl = &VariantChangeTypeVtbl;
+    ret->IWineJScript_iface.lpVtbl = &WineJScriptVtbl;
     ret->ref = 1;
     ret->safeopt = INTERFACE_USES_DISPEX;
     ret->is_encode = is_encode;
