@@ -1453,6 +1453,8 @@ static void test_session(void)
     WAVEFORMATEX *pwfx;
     ULONG ref;
     HRESULT hr;
+    WCHAR *str;
+    GUID guid1 = GUID_NULL, guid2 = GUID_NULL;
 
     hr = CoCreateGuid(&ses1_guid);
     ok(hr == S_OK, "CoCreateGuid failed: %08lx\n", hr);
@@ -1575,6 +1577,80 @@ static void test_session(void)
     ok(hr == S_OK, "GetState failed: %08lx\n", hr);
     ok(state == AudioSessionStateInactive, "Got wrong state: %d\n", state);
 
+    /* Test GetDisplayName / SetDisplayName */
+
+    hr = IAudioSessionControl2_GetDisplayName(ses1_ctl2, NULL);
+    ok(hr == E_POINTER, "GetDisplayName failed: %08lx\n", hr);
+
+    str = NULL;
+    hr = IAudioSessionControl2_GetDisplayName(ses1_ctl2, &str);
+    ok(hr == S_OK, "GetDisplayName failed: %08lx\n", hr);
+    ok(str && !wcscmp(str, L""), "Got %s\n", wine_dbgstr_w(str));
+    if (str)
+        CoTaskMemFree(str);
+
+    hr = IAudioSessionControl2_SetDisplayName(ses1_ctl2, NULL, NULL);
+    ok(hr == HRESULT_FROM_WIN32(RPC_X_NULL_REF_POINTER), "SetDisplayName failed: %08lx\n", hr);
+
+    hr = IAudioSessionControl2_SetDisplayName(ses1_ctl2, L"WineDisplayName", NULL);
+    ok(hr == S_OK, "SetDisplayName failed: %08lx\n", hr);
+
+    str = NULL;
+    hr = IAudioSessionControl2_GetDisplayName(ses1_ctl2, &str);
+    ok(hr == S_OK, "GetDisplayName failed: %08lx\n", hr);
+    ok(str && !wcscmp(str, L"WineDisplayName"), "Got %s\n", wine_dbgstr_w(str));
+    if (str)
+        CoTaskMemFree(str);
+
+    /* Test GetIconPath / SetIconPath */
+
+    hr = IAudioSessionControl2_GetIconPath(ses1_ctl2, NULL);
+    ok(hr == E_POINTER, "GetIconPath failed: %08lx\n", hr);
+
+    str = NULL;
+    hr = IAudioSessionControl2_GetIconPath(ses1_ctl2, &str);
+    ok(hr == S_OK, "GetIconPath failed: %08lx\n", hr);
+    ok(str && !wcscmp(str, L""), "Got %s\n", wine_dbgstr_w(str));
+    if(str)
+        CoTaskMemFree(str);
+
+    hr = IAudioSessionControl2_SetIconPath(ses1_ctl2, NULL, NULL);
+    ok(hr == HRESULT_FROM_WIN32(RPC_X_NULL_REF_POINTER), "SetIconPath failed: %08lx\n", hr);
+
+    hr = IAudioSessionControl2_SetIconPath(ses1_ctl2, L"WineIconPath", NULL);
+    ok(hr == S_OK, "SetIconPath failed: %08lx\n", hr);
+
+    str = NULL;
+    hr = IAudioSessionControl2_GetIconPath(ses1_ctl2, &str);
+    ok(hr == S_OK, "GetIconPath failed: %08lx\n", hr);
+    ok(str && !wcscmp(str, L"WineIconPath"), "Got %s\n", wine_dbgstr_w(str));
+    if (str)
+        CoTaskMemFree(str);
+
+    /* Test GetGroupingParam / SetGroupingParam */
+
+    hr = IAudioSessionControl2_GetGroupingParam(ses1_ctl2, NULL);
+    ok(hr == HRESULT_FROM_WIN32(RPC_X_NULL_REF_POINTER), "GetGroupingParam failed: %08lx\n", hr);
+
+    hr = IAudioSessionControl2_GetGroupingParam(ses1_ctl2, &guid1);
+    ok(hr == S_OK, "GetGroupingParam failed: %08lx\n", hr);
+    ok(!IsEqualGUID(&guid1, &guid2), "Expected non null GUID\n"); /* MSDN is wrong here, it is not GUID_NULL */
+
+    hr = IAudioSessionControl2_SetGroupingParam(ses1_ctl2, NULL, NULL);
+    ok(hr == HRESULT_FROM_WIN32(RPC_X_NULL_REF_POINTER), "SetGroupingParam failed: %08lx\n", hr);
+
+    hr = CoCreateGuid(&guid2);
+    ok(hr == S_OK, "CoCreateGuid failed: %08lx\n", hr);
+
+    hr = IAudioSessionControl2_SetGroupingParam(ses1_ctl2, &guid2, NULL);
+    ok(hr == S_OK, "SetGroupingParam failed: %08lx\n", hr);
+
+    hr = IAudioSessionControl2_GetGroupingParam(ses1_ctl2, &guid1);
+    ok(hr == S_OK, "GetGroupingParam failed: %08lx\n", hr);
+    ok(IsEqualGUID(&guid1, &guid2), "Got %s\n", wine_dbgstr_guid(&guid1));
+
+    /* Test capture */
+
     if(cap_ctl){
         hr = IAudioSessionControl2_GetState(cap_ctl, &state);
         ok(hr == S_OK, "GetState failed: %08lx\n", hr);
