@@ -536,7 +536,7 @@ static inline BOOL can_activate_window( HWND hwnd )
     if (style & WS_MINIMIZE) return FALSE;
     if (NtUserGetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_NOACTIVATE) return FALSE;
     if (hwnd == NtUserGetDesktopWindow()) return FALSE;
-    if (NtUserGetWindowRect( hwnd, &rect ) && IsRectEmpty( &rect )) return FALSE;
+    if (NtUserGetWindowRect( hwnd, &rect, get_win_monitor_dpi( hwnd ) ) && IsRectEmpty( &rect )) return FALSE;
     return !(style & WS_DISABLED);
 }
 
@@ -772,7 +772,7 @@ static BOOL X11DRV_FocusIn( HWND hwnd, XEvent *xev )
 
     if (event->detail == NotifyPointer) return FALSE;
     /* when focusing in the virtual desktop window, re-apply the cursor clipping rect */
-    if (is_virtual_desktop() && hwnd == NtUserGetDesktopWindow()) retry_grab_clipping_window();
+    if (is_virtual_desktop() && hwnd == NtUserGetDesktopWindow()) reapply_cursor_clipping();
     if (hwnd == NtUserGetDesktopWindow()) return FALSE;
 
     x11drv_thread_data()->keymapnotify_hwnd = hwnd;
@@ -780,7 +780,7 @@ static BOOL X11DRV_FocusIn( HWND hwnd, XEvent *xev )
     /* when keyboard grab is released, re-apply the cursor clipping rect */
     was_grabbed = keyboard_grabbed;
     keyboard_grabbed = event->mode == NotifyGrab || event->mode == NotifyWhileGrabbed;
-    if (was_grabbed > keyboard_grabbed) retry_grab_clipping_window();
+    if (was_grabbed > keyboard_grabbed) reapply_cursor_clipping();
     /* ignore wm specific NotifyUngrab / NotifyGrab events w.r.t focus */
     if (event->mode == NotifyGrab || event->mode == NotifyUngrab) return FALSE;
 
@@ -917,15 +917,12 @@ static BOOL X11DRV_Expose( HWND hwnd, XEvent *xev )
         if (NtUserGetWindowLongW( data->hwnd, GWL_EXSTYLE ) & WS_EX_LAYOUTRTL)
             mirror_rect( &data->client_rect, &rect );
         abs_rect = rect;
-        NtUserMapWindowPoints( hwnd, 0, (POINT *)&abs_rect, 2 );
+        NtUserMapWindowPoints( hwnd, 0, (POINT *)&abs_rect, 2, 0 /* per-monitor DPI */ );
 
         SERVER_START_REQ( update_window_zorder )
         {
             req->window      = wine_server_user_handle( hwnd );
-            req->rect.left   = abs_rect.left;
-            req->rect.top    = abs_rect.top;
-            req->rect.right  = abs_rect.right;
-            req->rect.bottom = abs_rect.bottom;
+            req->rect        = wine_server_rectangle( abs_rect );
             wine_server_call( req );
         }
         SERVER_END_REQ;
@@ -934,7 +931,7 @@ static BOOL X11DRV_Expose( HWND hwnd, XEvent *xev )
 
     release_win_data( data );
 
-    if (flags) NtUserRedrawWindow( hwnd, &rect, surface_region, flags );
+    if (flags) redraw_window( hwnd, &rect, surface_region, flags );
     if (surface_region) NtGdiDeleteObjectApp( surface_region );
     return TRUE;
 }
@@ -977,7 +974,7 @@ static BOOL X11DRV_UnmapNotify( HWND hwnd, XEvent *event )
 static void reparent_notify( Display *display, HWND hwnd, Window xparent, int x, int y )
 {
     HWND parent, old_parent;
-    DWORD style;
+    DWORD style, flags = 0;
 
     style = NtUserGetWindowLongW( hwnd, GWL_STYLE );
     if (xparent == root_window)
@@ -994,9 +991,9 @@ static void reparent_notify( Display *display, HWND hwnd, Window xparent, int x,
     NtUserShowWindow( hwnd, SW_HIDE );
     old_parent = NtUserSetParent( hwnd, parent );
     NtUserSetWindowLong( hwnd, GWL_STYLE, style, FALSE );
-    NtUserSetWindowPos( hwnd, HWND_TOP, x, y, 0, 0,
-                        SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOCOPYBITS |
-                        ((style & WS_VISIBLE) ? SWP_SHOWWINDOW : 0) );
+
+    if (style & WS_VISIBLE) flags = SWP_SHOWWINDOW;
+    set_window_pos( hwnd, HWND_TOP, x, y, 0, 0, SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOCOPYBITS | flags );
 
     /* make old parent destroy itself if it no longer has children */
     if (old_parent != NtUserGetDesktopWindow()) NtUserPostMessage( old_parent, WM_CLOSE, 0, 0 );
@@ -1049,7 +1046,7 @@ static BOOL X11DRV_ConfigureNotify( HWND hwnd, XEvent *xev )
     struct x11drv_win_data *data;
     RECT rect;
     POINT pos;
-    UINT flags;
+    UINT flags, dpi;
     HWND parent;
     BOOL root_coords;
     int cx, cy, x = event->x, y = event->y;
@@ -1071,6 +1068,7 @@ static BOOL X11DRV_ConfigureNotify( HWND hwnd, XEvent *xev )
 
     /* Get geometry */
 
+    dpi = get_win_monitor_dpi( data->hwnd );
     parent = NtUserGetAncestor( hwnd, GA_PARENT );
     root_coords = event->send_event;  /* synthetic events are always in root coords */
 
@@ -1090,7 +1088,7 @@ static BOOL X11DRV_ConfigureNotify( HWND hwnd, XEvent *xev )
     else pos = root_to_virtual_screen( x, y );
 
     X11DRV_X_to_window_rect( data, &rect, pos.x, pos.y, event->width, event->height );
-    if (root_coords) NtUserMapWindowPoints( 0, parent, (POINT *)&rect, 2 );
+    if (root_coords) NtUserMapWindowPoints( 0, parent, (POINT *)&rect, 2, 0 /* per-monitor DPI */ );
 
     TRACE( "win %p/%lx new X rect %d,%d,%dx%d (event %d,%d,%dx%d)\n",
            hwnd, data->whole_window, (int)rect.left, (int)rect.top,
@@ -1122,7 +1120,7 @@ static BOOL X11DRV_ConfigureNotify( HWND hwnd, XEvent *xev )
                (int)(data->window_rect.bottom - data->window_rect.top), cx, cy );
 
     style = NtUserGetWindowLongW( data->hwnd, GWL_STYLE );
-    if ((style & WS_CAPTION) == WS_CAPTION || !NtUserIsWindowRectFullScreen( &data->whole_rect ))
+    if ((style & WS_CAPTION) == WS_CAPTION || !NtUserIsWindowRectFullScreen( &data->whole_rect, dpi ))
     {
         read_net_wm_states( event->display, data );
         if ((data->net_wm_state & (1 << NET_WM_STATE_MAXIMIZED)))
@@ -1147,7 +1145,7 @@ static BOOL X11DRV_ConfigureNotify( HWND hwnd, XEvent *xev )
     if ((flags & (SWP_NOSIZE | SWP_NOMOVE)) != (SWP_NOSIZE | SWP_NOMOVE))
     {
         release_win_data( data );
-        NtUserSetWindowPos( hwnd, 0, x, y, cx, cy, flags );
+        set_window_pos( hwnd, 0, x, y, cx, cy, flags );
         return TRUE;
     }
 
@@ -1185,7 +1183,7 @@ static BOOL X11DRV_GravityNotify( HWND hwnd, XEvent *xev )
     release_win_data( data );
 
     if (window_rect.left != x || window_rect.top != y)
-        NtUserSetWindowPos( hwnd, 0, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOCOPYBITS );
+        set_window_pos( hwnd, 0, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOCOPYBITS );
 
     return TRUE;
 }
@@ -1408,24 +1406,24 @@ void X11DRV_SetFocus( HWND hwnd )
 
 static HWND find_drop_window( HWND hQueryWnd, LPPOINT lpPt )
 {
+    UINT dpi = get_win_monitor_dpi( hQueryWnd );
     RECT tempRect;
 
     if (!NtUserIsWindowEnabled(hQueryWnd)) return 0;
     
-    NtUserGetWindowRect(hQueryWnd, &tempRect);
+    NtUserGetWindowRect( hQueryWnd, &tempRect, dpi );
 
     if(!PtInRect(&tempRect, *lpPt)) return 0;
 
     if (!(NtUserGetWindowLongW( hQueryWnd, GWL_STYLE ) & WS_MINIMIZE))
     {
         POINT pt = *lpPt;
-        NtUserScreenToClient( hQueryWnd, &pt );
-        NtUserGetClientRect( hQueryWnd, &tempRect );
+        NtUserMapWindowPoints( 0, hQueryWnd, &pt, 1, 0 /* per-monitor DPI */ );
+        NtUserGetClientRect( hQueryWnd, &tempRect, dpi );
 
         if (PtInRect( &tempRect, pt))
         {
-            HWND ret = NtUserChildWindowFromPointEx( hQueryWnd, pt.x, pt.y,
-                                                     CWP_SKIPINVISIBLE|CWP_SKIPDISABLED );
+            HWND ret = child_window_from_point( hQueryWnd, pt.x, pt.y, CWP_SKIPINVISIBLE | CWP_SKIPDISABLED );
             if (ret && ret != hQueryWnd)
             {
                 ret = find_drop_window( ret, lpPt );
@@ -1436,7 +1434,7 @@ static HWND find_drop_window( HWND hQueryWnd, LPPOINT lpPt )
 
     if(!(NtUserGetWindowLongW( hQueryWnd, GWL_EXSTYLE ) & WS_EX_ACCEPTFILES)) return 0;
     
-    NtUserScreenToClient( hQueryWnd, lpPt );
+    NtUserMapWindowPoints( 0, hQueryWnd, lpPt, 1, 0 /* per-monitor DPI */ );
 
     return hQueryWnd;
 }
